@@ -17,6 +17,9 @@ log = logging.getLogger('vkontakte_groups_migration')
 
 FETCH_ONLY_EXPIRED_USERS = getattr(settings, 'VKONTAKTE_GROUPS_MIGRATION_FETCH_ONLY_EXPIRED_USERS', True)
 
+class WrongMembershipsAmmount(Exception):
+    pass
+
 def update_group_users(migration):
     '''
     Fetch all users of group, make new m2m relations, remove old m2m relations
@@ -36,12 +39,10 @@ def update_group_users(migration):
     ids_entered = set(migration.members_ids).difference(set(ids_current))
 
     log.debug('Adding %d new users to the group "%s"' % (len(ids_entered), group))
-    ids = User.objects.filter(remote_id__in=ids_entered).values_list('pk', flat=True)
-    group.users.through.objects.bulk_create([group.users.through(group=group, user_id=id) for id in ids])
+    group.users.through.objects.bulk_create([group.users.through(group=group, user_id=id) for id in ids_entered])
 
     log.info('Removing %d left users from the group "%s"' % (len(ids_left), group))
-    ids = User.objects.filter(remote_id__in=ids_left).values_list('pk', flat=True)
-    group.users.through.objects.filter(group=group, user_id__in=ids).delete()
+    group.users.through.objects.filter(group=group, user_id__in=ids_left).delete()
 
     signals.group_users_updated.send(sender=Group, instance=group)
     log.info('Updating m2m relations of users for group "%s" successfuly finished' % group)
@@ -270,6 +271,7 @@ class GroupMigration(models.Model):
             next_stat.update()
             next_stat.save()
 
+    @transaction.commit_on_success
     def save(self, *args, **kwargs):
         try:
             assert self.hidden != self.__class__.objects.light.get(pk=self.pk).hidden
@@ -292,8 +294,18 @@ class GroupMigration(models.Model):
         self.clean_members()
         self.update()
         self.save()
+
+        # check if migration is correct
+        self.compare_with_statistic()
+        self.compare_entered_left()
+        self.compare_with_siblings()
+
         # call only after saving migrations, because in case of fault we need to have right migrations as source for memberships
-        self.update_users_memberships()
+        try:
+            self.update_users_memberships()
+        except WrongMembershipsAmmount, e:
+            log.warning(e)
+            GroupMembership.objects.fix_timeline(group=self)
 
     def compare_with_siblings(self):
         if self.hidden or not self.prev or self.members_count < 10000:
@@ -399,10 +411,7 @@ class GroupMigration(models.Model):
             members_count = self.prev.members_count
             if members_count != memberships_count:
                 # incostistent state with previous migration
-                raise Exception("Number of current memberships %d is not equal to members count %d of previous migration, group %s at %s" % (memberships_count, members_count, self.group, self.time))
-#                log.warning("Number of current memberships %d is not equal to members count %d of previous migration, group %s at %s" % (memberships_count, members_count, self.group, self.time))
-#                GroupMembership.objects.fix_timeline(self.group)
-#                return True
+                raise WrongMembershipsAmmount("Number of current memberships %d is not equal to members count %d of previous migration, group %s at %s" % (memberships_count, members_count, self.group, self.time))
 
             # ensure entered users not in memberships now
             error_ids_count = GroupMembership.objects.get_user_ids(self.group).filter(user_id__in=self.members_entered_ids).count()
@@ -506,7 +515,6 @@ class GroupMembership(models.Model):
     class Meta:
         verbose_name = u'Членство пользователя группы Вконтакте'
         verbose_name_plural = u'Членства пользователей групп Вконтакте'
-#        unique_together = (('group','user_id','time_entered'), ('group','user_id','time_left'),)
 
     group = models.ForeignKey(Group, verbose_name=u'Группа', related_name='memberships')
     user_id = models.PositiveIntegerField(u'ID пользователя', db_index=True)
